@@ -9,14 +9,15 @@
  */
 namespace SebastianBergmann\CodeCoverage\Report;
 
+use const DIRECTORY_SEPARATOR;
 use function basename;
 use function count;
 use function dirname;
 use function file_put_contents;
 use function preg_match;
 use function range;
+use function str_contains;
 use function str_replace;
-use function strpos;
 use function time;
 use DOMImplementation;
 use SebastianBergmann\CodeCoverage\CodeCoverage;
@@ -40,7 +41,7 @@ final class Cobertura
         $documentType = $implementation->createDocumentType(
             'coverage',
             '',
-            'http://cobertura.sourceforge.net/xml/coverage-04.dtd'
+            'http://cobertura.sourceforge.net/xml/coverage-04.dtd',
         );
 
         $document               = $implementation->createDocument('', '', $documentType);
@@ -114,13 +115,9 @@ final class Cobertura
             $coverageData = $item->lineCoverageData();
 
             foreach ($classes as $className => $class) {
-                $complexity += $class['ccn'];
+                $complexity        += $class['ccn'];
                 $packageComplexity += $class['ccn'];
 
-                if (!empty($class['package']['namespace'])) {
-                    $className = $class['package']['namespace'] . '\\' . $className;
-                }
-
                 $linesValid   = $class['executableLines'];
                 $linesCovered = $class['executedLines'];
                 $lineRate     = $linesValid === 0 ? 0 : ($linesCovered / $linesValid);
@@ -175,7 +172,7 @@ final class Cobertura
                     $methodElement->appendChild($methodLinesElement);
 
                     foreach (range($method['startLine'], $method['endLine']) as $line) {
-                        if (!isset($coverageData[$line]) || $coverageData[$line] === null) {
+                        if (!isset($coverageData[$line])) {
                             continue;
                         }
                         $methodLineElement = $document->createElement('line');
@@ -225,22 +222,22 @@ final class Cobertura
                     continue;
                 }
 
-                $complexity += $function['ccn'];
-                $packageComplexity += $function['ccn'];
+                $complexity          += $function['ccn'];
+                $packageComplexity   += $function['ccn'];
                 $functionsComplexity += $function['ccn'];
 
                 $linesValid   = $function['executableLines'];
                 $linesCovered = $function['executedLines'];
                 $lineRate     = $linesValid === 0 ? 0 : ($linesCovered / $linesValid);
 
-                $functionsLinesValid += $linesValid;
+                $functionsLinesValid   += $linesValid;
                 $functionsLinesCovered += $linesCovered;
 
                 $branchesValid   = $function['executableBranches'];
                 $branchesCovered = $function['executedBranches'];
                 $branchRate      = $branchesValid === 0 ? 0 : ($branchesCovered / $branchesValid);
 
-                $functionsBranchesValid += $branchesValid;
+                $functionsBranchesValid   += $branchesValid;
                 $functionsBranchesCovered += $branchesValid;
 
                 $methodElement = $document->createElement('method');
@@ -256,7 +253,7 @@ final class Cobertura
                 $methodElement->appendChild($methodLinesElement);
 
                 foreach (range($function['startLine'], $function['endLine']) as $line) {
-                    if (!isset($coverageData[$line]) || $coverageData[$line] === null) {
+                    if (!isset($coverageData[$line])) {
                         continue;
                     }
                     $methodLineElement = $document->createElement('line');
@@ -295,7 +292,7 @@ final class Cobertura
         $buffer = $document->saveXML();
 
         if ($target !== null) {
-            if (!strpos($target, '://') !== false) {
+            if (!str_contains($target, '://')) {
                 Filesystem::createDirectory(dirname($target));
             }
 
