@@ -38,11 +38,13 @@ use const T_ENDFOREACH;
 use const T_ENDIF;
 use const T_ENDSWITCH;
 use const T_ENDWHILE;
+use const T_ENUM;
 use const T_EVAL;
 use const T_EXIT;
 use const T_EXTENDS;
 use const T_FINAL;
 use const T_FINALLY;
+use const T_FN;
 use const T_FOR;
 use const T_FOREACH;
 use const T_FUNCTION;
@@ -59,12 +61,14 @@ use const T_INSTEADOF;
 use const T_INTERFACE;
 use const T_ISSET;
 use const T_LIST;
+use const T_MATCH;
 use const T_NAMESPACE;
 use const T_NEW;
 use const T_PRINT;
 use const T_PRIVATE;
 use const T_PROTECTED;
 use const T_PUBLIC;
+use const T_READONLY;
 use const T_REQUIRE;
 use const T_REQUIRE_ONCE;
 use const T_RETURN;
@@ -80,23 +84,27 @@ use const T_WHILE;
 use const T_YIELD;
 use const T_YIELD_FROM;
 use function array_key_exists;
+use function array_keys;
+use function array_merge;
 use function array_pop;
 use function array_unique;
-use function constant;
 use function count;
-use function defined;
 use function explode;
 use function file_get_contents;
 use function htmlspecialchars;
 use function is_string;
+use function ksort;
+use function range;
+use function sort;
 use function sprintf;
+use function str_ends_with;
 use function str_replace;
-use function substr;
 use function token_get_all;
 use function trim;
-use PHPUnit\Runner\BaseTestRunner;
+use SebastianBergmann\CodeCoverage\FileCouldNotBeWrittenException;
 use SebastianBergmann\CodeCoverage\Node\File as FileNode;
-use SebastianBergmann\CodeCoverage\Percentage;
+use SebastianBergmann\CodeCoverage\Util\Percentage;
+use SebastianBergmann\Template\Exception;
 use SebastianBergmann\Template\Template;
 
 /**
@@ -105,19 +113,80 @@ use SebastianBergmann\Template\Template;
 final class File extends Renderer
 {
     /**
-     * @psalm-var array<int,true>
+     * @var array<int,true>
      */
-    private static $keywordTokens = [];
-
-    /**
-     * @var array
-     */
-    private static $formattedSourceCache = [];
-
-    /**
-     * @var int
-     */
-    private $htmlSpecialCharsFlags = ENT_COMPAT | ENT_HTML401 | ENT_SUBSTITUTE;
+    private const KEYWORD_TOKENS = [
+        T_ABSTRACT      => true,
+        T_ARRAY         => true,
+        T_AS            => true,
+        T_BREAK         => true,
+        T_CALLABLE      => true,
+        T_CASE          => true,
+        T_CATCH         => true,
+        T_CLASS         => true,
+        T_CLONE         => true,
+        T_CONST         => true,
+        T_CONTINUE      => true,
+        T_DECLARE       => true,
+        T_DEFAULT       => true,
+        T_DO            => true,
+        T_ECHO          => true,
+        T_ELSE          => true,
+        T_ELSEIF        => true,
+        T_EMPTY         => true,
+        T_ENDDECLARE    => true,
+        T_ENDFOR        => true,
+        T_ENDFOREACH    => true,
+        T_ENDIF         => true,
+        T_ENDSWITCH     => true,
+        T_ENDWHILE      => true,
+        T_ENUM          => true,
+        T_EVAL          => true,
+        T_EXIT          => true,
+        T_EXTENDS       => true,
+        T_FINAL         => true,
+        T_FINALLY       => true,
+        T_FN            => true,
+        T_FOR           => true,
+        T_FOREACH       => true,
+        T_FUNCTION      => true,
+        T_GLOBAL        => true,
+        T_GOTO          => true,
+        T_HALT_COMPILER => true,
+        T_IF            => true,
+        T_IMPLEMENTS    => true,
+        T_INCLUDE       => true,
+        T_INCLUDE_ONCE  => true,
+        T_INSTANCEOF    => true,
+        T_INSTEADOF     => true,
+        T_INTERFACE     => true,
+        T_ISSET         => true,
+        T_LIST          => true,
+        T_MATCH         => true,
+        T_NAMESPACE     => true,
+        T_NEW           => true,
+        T_PRINT         => true,
+        T_PRIVATE       => true,
+        T_PROTECTED     => true,
+        T_PUBLIC        => true,
+        T_READONLY      => true,
+        T_REQUIRE       => true,
+        T_REQUIRE_ONCE  => true,
+        T_RETURN        => true,
+        T_STATIC        => true,
+        T_SWITCH        => true,
+        T_THROW         => true,
+        T_TRAIT         => true,
+        T_TRY           => true,
+        T_UNSET         => true,
+        T_USE           => true,
+        T_VAR           => true,
+        T_WHILE         => true,
+        T_YIELD         => true,
+        T_YIELD_FROM    => true,
+    ];
+    private static array $formattedSourceCache = [];
+    private int $htmlSpecialCharsFlags         = ENT_COMPAT | ENT_HTML401 | ENT_SUBSTITUTE;
 
     public function render(FileNode $node, string $file): void
     {
@@ -129,12 +198,20 @@ final class File extends Renderer
             [
                 'items'     => $this->renderItems($node),
                 'lines'     => $this->renderSourceWithLineCoverage($node),
-                'legend'    => '<p><span class="success"><strong>Executed</strong></span><span class="danger"><strong>Not Executed</strong></span><span class="warning"><strong>Dead Code</strong></span></p>',
+                'legend'    => '<p><span class="legend covered-by-small-tests">Covered by small (and larger) tests</span><span class="legend covered-by-medium-tests">Covered by medium (and large) tests</span><span class="legend covered-by-large-tests">Covered by large tests (and tests of unknown size)</span><span class="legend not-covered">Not covered</span><span class="legend not-coverable">Not coverable</span></p>',
                 'structure' => '',
-            ]
+            ],
         );
 
-        $template->renderTo($file . '.html');
+        try {
+            $template->renderTo($file . '.html');
+        } catch (Exception $e) {
+            throw new FileCouldNotBeWrittenException(
+                $e->getMessage(),
+                $e->getCode(),
+                $e,
+            );
+        }
 
         if ($this->hasBranchCoverage) {
             $template->setVar(
@@ -143,10 +220,18 @@ final class File extends Renderer
                     'lines'     => $this->renderSourceWithBranchCoverage($node),
                     'legend'    => '<p><span class="success"><strong>Fully covered</strong></span><span class="warning"><strong>Partially covered</strong></span><span class="danger"><strong>Not covered</strong></span></p>',
                     'structure' => $this->renderBranchStructure($node),
-                ]
+                ],
             );
 
-            $template->renderTo($file . '_branch.html');
+            try {
+                $template->renderTo($file . '_branch.html');
+            } catch (Exception $e) {
+                throw new FileCouldNotBeWrittenException(
+                    $e->getMessage(),
+                    $e->getCode(),
+                    $e,
+                );
+            }
 
             $template->setVar(
                 [
@@ -154,10 +239,18 @@ final class File extends Renderer
                     'lines'     => $this->renderSourceWithPathCoverage($node),
                     'legend'    => '<p><span class="success"><strong>Fully covered</strong></span><span class="warning"><strong>Partially covered</strong></span><span class="danger"><strong>Not covered</strong></span></p>',
                     'structure' => $this->renderPathStructure($node),
-                ]
+                ],
             );
 
-            $template->renderTo($file . '_path.html');
+            try {
+                $template->renderTo($file . '_path.html');
+            } catch (Exception $e) {
+                throw new FileCouldNotBeWrittenException(
+                    $e->getMessage(),
+                    $e->getCode(),
+                    $e,
+                );
+            }
         }
     }
 
@@ -170,7 +263,7 @@ final class File extends Renderer
         $methodItemTemplate = new Template(
             $methodTemplateName,
             '{{',
-            '}}'
+            '}}',
         );
 
         $items = $this->renderItemTemplate(
@@ -198,24 +291,24 @@ final class File extends Renderer
                 'testedClassesPercent'            => $node->percentageOfTestedClassesAndTraits()->asFloat(),
                 'testedClassesPercentAsString'    => $node->percentageOfTestedClassesAndTraits()->asString(),
                 'crap'                            => '<abbr title="Change Risk Anti-Patterns (CRAP) Index">CRAP</abbr>',
-            ]
+            ],
         );
 
         $items .= $this->renderFunctionItems(
             $node->functions(),
-            $methodItemTemplate
+            $methodItemTemplate,
         );
 
         $items .= $this->renderTraitOrClassItems(
             $node->traits(),
             $template,
-            $methodItemTemplate
+            $methodItemTemplate,
         );
 
         $items .= $this->renderTraitOrClassItems(
             $node->classes(),
             $template,
-            $methodItemTemplate
+            $methodItemTemplate,
         );
 
         return $items;
@@ -248,15 +341,15 @@ final class File extends Renderer
                 $numTestedClasses             = $numTestedMethods === $numMethods ? 1 : 0;
                 $linesExecutedPercentAsString = Percentage::fromFractionAndTotal(
                     $item['executedLines'],
-                    $item['executableLines']
+                    $item['executableLines'],
                 )->asString();
                 $branchesExecutedPercentAsString = Percentage::fromFractionAndTotal(
                     $item['executedBranches'],
-                    $item['executableBranches']
+                    $item['executableBranches'],
                 )->asString();
                 $pathsExecutedPercentAsString = Percentage::fromFractionAndTotal(
                     $item['executedPaths'],
-                    $item['executablePaths']
+                    $item['executablePaths'],
                 )->asString();
             } else {
                 $numClasses                      = 0;
@@ -268,12 +361,12 @@ final class File extends Renderer
 
             $testedMethodsPercentage = Percentage::fromFractionAndTotal(
                 $numTestedMethods,
-                $numMethods
+                $numMethods,
             );
 
             $testedClassesPercentage = Percentage::fromFractionAndTotal(
                 $numTestedMethods === $numMethods ? 1 : 0,
-                1
+                1,
             );
 
             $buffer .= $this->renderItemTemplate(
@@ -300,7 +393,7 @@ final class File extends Renderer
                     'numExecutableBranches'           => $item['executableBranches'],
                     'pathsExecutedPercent'            => Percentage::fromFractionAndTotal(
                         $item['executedPaths'],
-                        $item['executablePaths']
+                        $item['executablePaths'],
                     )->asFloat(),
                     'pathsExecutedPercentAsString' => $pathsExecutedPercentAsString,
                     'numExecutedPaths'             => $item['executedPaths'],
@@ -310,14 +403,14 @@ final class File extends Renderer
                     'testedClassesPercent'         => $testedClassesPercentage->asFloat(),
                     'testedClassesPercentAsString' => $testedClassesPercentage->asString(),
                     'crap'                         => $item['crap'],
-                ]
+                ],
             );
 
             foreach ($item['methods'] as $method) {
                 $buffer .= $this->renderFunctionOrMethodItem(
                     $methodItemTemplate,
                     $method,
-                    '&nbsp;'
+                    '&nbsp;',
                 );
             }
         }
@@ -336,7 +429,7 @@ final class File extends Renderer
         foreach ($functions as $function) {
             $buffer .= $this->renderFunctionOrMethodItem(
                 $template,
-                $function
+                $function,
             );
         }
 
@@ -358,22 +451,22 @@ final class File extends Renderer
 
         $executedLinesPercentage = Percentage::fromFractionAndTotal(
             $item['executedLines'],
-            $item['executableLines']
+            $item['executableLines'],
         );
 
         $executedBranchesPercentage = Percentage::fromFractionAndTotal(
             $item['executedBranches'],
-            $item['executableBranches']
+            $item['executableBranches'],
         );
 
         $executedPathsPercentage = Percentage::fromFractionAndTotal(
             $item['executedPaths'],
-            $item['executablePaths']
+            $item['executablePaths'],
         );
 
         $testedMethodsPercentage = Percentage::fromFractionAndTotal(
             $numTestedMethods,
-            1
+            1,
         );
 
         return $this->renderItemTemplate(
@@ -384,7 +477,7 @@ final class File extends Renderer
                     $indent,
                     $item['startLine'],
                     htmlspecialchars($item['signature'], $this->htmlSpecialCharsFlags),
-                    $item['functionName'] ?? $item['methodName']
+                    $item['functionName'] ?? $item['methodName'],
                 ),
                 'numMethods'                      => $numMethods,
                 'numTestedMethods'                => $numTestedMethods,
@@ -403,7 +496,7 @@ final class File extends Renderer
                 'testedMethodsPercent'            => $testedMethodsPercentage->asFloat(),
                 'testedMethodsPercentAsString'    => $testedMethodsPercentage->asString(),
                 'crap'                            => $item['crap'],
-            ]
+            ],
         );
     }
 
@@ -459,9 +552,9 @@ final class File extends Renderer
 
             if (!empty($popoverTitle)) {
                 $popover = sprintf(
-                    ' data-title="%s" data-content="%s" data-placement="top" data-html="true"',
+                    ' data-bs-title="%s" data-bs-content="%s" data-bs-placement="top" data-bs-html="true"',
                     $popoverTitle,
-                    htmlspecialchars($popoverContent, $this->htmlSpecialCharsFlags)
+                    htmlspecialchars($popoverContent, $this->htmlSpecialCharsFlags),
                 );
             }
 
@@ -546,9 +639,9 @@ final class File extends Renderer
                 $trClass = $lineCss . ' popin';
 
                 $popover = sprintf(
-                    ' data-title="%s" data-content="%s" data-placement="top" data-html="true"',
+                    ' data-bs-title="%s" data-bs-content="%s" data-bs-placement="top" data-bs-html="true"',
                     $popoverTitle,
-                    htmlspecialchars($popoverContent, $this->htmlSpecialCharsFlags)
+                    htmlspecialchars($popoverContent, $this->htmlSpecialCharsFlags),
                 );
             }
 
@@ -636,9 +729,9 @@ final class File extends Renderer
                 $trClass = $lineCss . ' popin';
 
                 $popover = sprintf(
-                    ' data-title="%s" data-content="%s" data-placement="top" data-html="true"',
+                    ' data-bs-title="%s" data-bs-content="%s" data-bs-placement="top" data-bs-html="true"',
                     $popoverTitle,
-                    htmlspecialchars($popoverContent, $this->htmlSpecialCharsFlags)
+                    htmlspecialchars($popoverContent, $this->htmlSpecialCharsFlags),
                 );
             }
 
@@ -734,9 +827,9 @@ final class File extends Renderer
 
             if (!empty($popoverTitle)) {
                 $popover = sprintf(
-                    ' data-title="%s" data-content="%s" data-placement="top" data-html="true"',
+                    ' data-bs-title="%s" data-bs-content="%s" data-bs-placement="top" data-bs-html="true"',
                     $popoverTitle,
-                    htmlspecialchars($popoverContent, $this->htmlSpecialCharsFlags)
+                    htmlspecialchars($popoverContent, $this->htmlSpecialCharsFlags),
                 );
             }
 
@@ -797,8 +890,15 @@ final class File extends Renderer
         $singleLineTemplate = new Template($this->templatePath . 'line.html.dist', '{{', '}}');
 
         $lines = '';
+        $first = true;
 
         foreach ($path['path'] as $branchId) {
+            if ($first) {
+                $first = false;
+            } else {
+                $lines .= '    <tr><td colspan="2">&nbsp;</td></tr>' . "\n";
+            }
+
             $branchLines = range($branches[$branchId]['line_start'], $branches[$branchId]['line_end']);
             sort($branchLines); // sometimes end_line < start_line
 
@@ -834,6 +934,7 @@ final class File extends Renderer
 
                         $popoverContent .= $this->createPopoverContentForTest($test, $testData[$test]);
                     }
+
                     $trClass = $lineCss . ' popin';
                 }
 
@@ -841,9 +942,9 @@ final class File extends Renderer
 
                 if (!empty($popoverTitle)) {
                     $popover = sprintf(
-                        ' data-title="%s" data-content="%s" data-placement="top" data-html="true"',
+                        ' data-bs-title="%s" data-bs-content="%s" data-bs-placement="top" data-bs-html="true"',
                         $popoverTitle,
-                        htmlspecialchars($popoverContent, $this->htmlSpecialCharsFlags)
+                        htmlspecialchars($popoverContent, $this->htmlSpecialCharsFlags),
                     );
                 }
 
@@ -868,7 +969,7 @@ final class File extends Renderer
                 'lineContent' => $lineContent,
                 'class'       => $class,
                 'popover'     => $popover,
-            ]
+            ],
         );
 
         return $template->render();
@@ -885,7 +986,7 @@ final class File extends Renderer
         $result              = [''];
         $i                   = 0;
         $stringFlag          = false;
-        $fileEndsWithNewLine = substr($buffer, -1) === "\n";
+        $fileEndsWithNewLine = str_ends_with($buffer, "\n");
 
         unset($buffer);
 
@@ -894,14 +995,14 @@ final class File extends Renderer
                 if ($token === '"' && $tokens[$j - 1] !== '\\') {
                     $result[$i] .= sprintf(
                         '<span class="string">%s</span>',
-                        htmlspecialchars($token, $this->htmlSpecialCharsFlags)
+                        htmlspecialchars($token, $this->htmlSpecialCharsFlags),
                     );
 
                     $stringFlag = !$stringFlag;
                 } else {
                     $result[$i] .= sprintf(
                         '<span class="keyword">%s</span>',
-                        htmlspecialchars($token, $this->htmlSpecialCharsFlags)
+                        htmlspecialchars($token, $this->htmlSpecialCharsFlags),
                     );
                 }
 
@@ -913,7 +1014,7 @@ final class File extends Renderer
             $value = str_replace(
                 ["\t", ' '],
                 ['&nbsp;&nbsp;&nbsp;&nbsp;', '&nbsp;'],
-                htmlspecialchars($value, $this->htmlSpecialCharsFlags)
+                htmlspecialchars($value, $this->htmlSpecialCharsFlags),
             );
 
             if ($value === "\n") {
@@ -942,7 +1043,7 @@ final class File extends Renderer
                         $result[$i] .= sprintf(
                             '<span class="%s">%s</span>',
                             $colour,
-                            $line
+                            $line,
                         );
                     }
 
@@ -970,7 +1071,7 @@ final class File extends Renderer
             $className = sprintf(
                 '<abbr title="%s">%s</abbr>',
                 $className,
-                array_pop($tmp)
+                array_pop($tmp),
             );
         }
 
@@ -992,48 +1093,27 @@ final class File extends Renderer
     {
         $testCSS = '';
 
-        if ($testData['fromTestcase']) {
-            switch ($testData['status']) {
-                case BaseTestRunner::STATUS_PASSED:
-                    switch ($testData['size']) {
-                        case 'small':
-                            $testCSS = ' class="covered-by-small-tests"';
+        switch ($testData['status']) {
+            case 'success':
+                $testCSS = match ($testData['size']) {
+                    'small'  => ' class="covered-by-small-tests"',
+                    'medium' => ' class="covered-by-medium-tests"',
+                    // no break
+                    default => ' class="covered-by-large-tests"',
+                };
 
-                            break;
+                break;
 
-                        case 'medium':
-                            $testCSS = ' class="covered-by-medium-tests"';
+            case 'failure':
+                $testCSS = ' class="danger"';
 
-                            break;
-
-                        default:
-                            $testCSS = ' class="covered-by-large-tests"';
-
-                            break;
-                    }
-
-                    break;
-
-                case BaseTestRunner::STATUS_SKIPPED:
-                case BaseTestRunner::STATUS_INCOMPLETE:
-                case BaseTestRunner::STATUS_RISKY:
-                case BaseTestRunner::STATUS_WARNING:
-                    $testCSS = ' class="warning"';
-
-                    break;
-
-                case BaseTestRunner::STATUS_FAILURE:
-                case BaseTestRunner::STATUS_ERROR:
-                    $testCSS = ' class="danger"';
-
-                    break;
-            }
+                break;
         }
 
         return sprintf(
             '<li%s>%s</li>',
             $testCSS,
-            htmlspecialchars($test, $this->htmlSpecialCharsFlags)
+            htmlspecialchars($test, $this->htmlSpecialCharsFlags),
         );
     }
 
@@ -1049,101 +1129,6 @@ final class File extends Renderer
 
     private function isKeyword(int $token): bool
     {
-        return isset(self::keywordTokens()[$token]);
-    }
-
-    /**
-     * @psalm-return array<int,true>
-     */
-    private static function keywordTokens(): array
-    {
-        if (self::$keywordTokens !== []) {
-            return self::$keywordTokens;
-        }
-
-        self::$keywordTokens = [
-            T_ABSTRACT      => true,
-            T_ARRAY         => true,
-            T_AS            => true,
-            T_BREAK         => true,
-            T_CALLABLE      => true,
-            T_CASE          => true,
-            T_CATCH         => true,
-            T_CLASS         => true,
-            T_CLONE         => true,
-            T_CONST         => true,
-            T_CONTINUE      => true,
-            T_DECLARE       => true,
-            T_DEFAULT       => true,
-            T_DO            => true,
-            T_ECHO          => true,
-            T_ELSE          => true,
-            T_ELSEIF        => true,
-            T_EMPTY         => true,
-            T_ENDDECLARE    => true,
-            T_ENDFOR        => true,
-            T_ENDFOREACH    => true,
-            T_ENDIF         => true,
-            T_ENDSWITCH     => true,
-            T_ENDWHILE      => true,
-            T_EVAL          => true,
-            T_EXIT          => true,
-            T_EXTENDS       => true,
-            T_FINAL         => true,
-            T_FINALLY       => true,
-            T_FOR           => true,
-            T_FOREACH       => true,
-            T_FUNCTION      => true,
-            T_GLOBAL        => true,
-            T_GOTO          => true,
-            T_HALT_COMPILER => true,
-            T_IF            => true,
-            T_IMPLEMENTS    => true,
-            T_INCLUDE       => true,
-            T_INCLUDE_ONCE  => true,
-            T_INSTANCEOF    => true,
-            T_INSTEADOF     => true,
-            T_INTERFACE     => true,
-            T_ISSET         => true,
-            T_LIST          => true,
-            T_NAMESPACE     => true,
-            T_NEW           => true,
-            T_PRINT         => true,
-            T_PRIVATE       => true,
-            T_PROTECTED     => true,
-            T_PUBLIC        => true,
-            T_REQUIRE       => true,
-            T_REQUIRE_ONCE  => true,
-            T_RETURN        => true,
-            T_STATIC        => true,
-            T_SWITCH        => true,
-            T_THROW         => true,
-            T_TRAIT         => true,
-            T_TRY           => true,
-            T_UNSET         => true,
-            T_USE           => true,
-            T_VAR           => true,
-            T_WHILE         => true,
-            T_YIELD         => true,
-            T_YIELD_FROM    => true,
-        ];
-
-        if (defined('T_FN')) {
-            self::$keywordTokens[constant('T_FN')] = true;
-        }
-
-        if (defined('T_MATCH')) {
-            self::$keywordTokens[constant('T_MATCH')] = true;
-        }
-
-        if (defined('T_ENUM')) {
-            self::$keywordTokens[constant('T_ENUM')] = true;
-        }
-
-        if (defined('T_READONLY')) {
-            self::$keywordTokens[constant('T_READONLY')] = true;
-        }
-
-        return self::$keywordTokens;
+        return isset(self::KEYWORD_TOKENS[$token]);
     }
 }
