@@ -12,54 +12,61 @@ namespace SebastianBergmann\CodeCoverage\StaticAnalysis;
 use function explode;
 use function file_get_contents;
 use function preg_match;
-use function strpos;
+use function str_contains;
 use PhpParser\NodeTraverser;
 use PhpParser\ParserFactory;
+use PHPUnit\Framework\Attributes\CoversClass;
+use PHPUnit\Framework\Attributes\RequiresPhp;
+use PHPUnit\Framework\Attributes\Ticket;
 use PHPUnit\Framework\TestCase;
 
-/**
- * @covers \SebastianBergmann\CodeCoverage\StaticAnalysis\ExecutableLinesFindingVisitor
- */
+#[CoversClass(ExecutableLinesFindingVisitor::class)]
 final class ExecutableLinesFindingVisitorTest extends TestCase
 {
     public function testExecutableLinesAreGroupedByBranch(): void
     {
-        $this->doTestSelfDescribingAsset(TEST_FILES_PATH . 'source_for_branched_exec_lines.php');
+        $this->doTestSelfDescribingAssert(TEST_FILES_PATH . 'source_for_branched_exec_lines.php');
     }
 
-    /**
-     * @requires PHP 7.4
-     */
-    public function testExecutableLinesAreGroupedByBranchPhp74(): void
-    {
-        $this->doTestSelfDescribingAsset(TEST_FILES_PATH . 'source_for_branched_exec_lines_php74.php');
-    }
-
-    /**
-     * @requires PHP 8
-     */
-    public function testExecutableLinesAreGroupedByBranchPhp80(): void
-    {
-        $this->doTestSelfDescribingAsset(TEST_FILES_PATH . 'source_for_branched_exec_lines_php80.php');
-    }
-
-    /**
-     * @requires PHP 8.1
-     */
+    #[RequiresPhp('>=8.1')]
     public function testExecutableLinesAreGroupedByBranchPhp81(): void
     {
-        $this->doTestSelfDescribingAsset(TEST_FILES_PATH . 'source_for_branched_exec_lines_php81.php');
+        $this->doTestSelfDescribingAssert(TEST_FILES_PATH . 'source_for_branched_exec_lines_php81.php');
     }
 
-    /**
-     * @requires PHP 8.2
-     */
+    #[RequiresPhp('>=8.2')]
     public function testExecutableLinesAreGroupedByBranchPhp82(): void
     {
-        $this->doTestSelfDescribingAsset(TEST_FILES_PATH . 'source_for_branched_exec_lines_php82.php');
+        $this->doTestSelfDescribingAssert(TEST_FILES_PATH . 'source_for_branched_exec_lines_php82.php');
     }
 
-    private function doTestSelfDescribingAsset(string $filename): void
+    #[Ticket('https://github.com/sebastianbergmann/php-code-coverage/issues/967')]
+    public function testMatchArmsAreProcessedCorrectly(): void
+    {
+        $source                        = file_get_contents(__DIR__ . '/../../_files/source_match_expression.php');
+        $parser                        = (new ParserFactory)->createForHostVersion();
+        $nodes                         = $parser->parse($source);
+        $executableLinesFindingVisitor = new ExecutableLinesFindingVisitor($source);
+
+        $traverser = new NodeTraverser;
+        $traverser->addVisitor($executableLinesFindingVisitor);
+        $traverser->traverse($nodes);
+
+        $this->assertSame(
+            [
+                8  => 2,
+                9  => 3,
+                10 => 4,
+                11 => 5,
+                12 => 6,
+                13 => 7,
+                14 => 2,
+            ],
+            $executableLinesFindingVisitor->executableLinesGroupedByBranch(),
+        );
+    }
+
+    private function doTestSelfDescribingAssert(string $filename): void
     {
         $source                        = file_get_contents($filename);
         $parser                        = (new ParserFactory)->createForHostVersion();
@@ -78,7 +85,7 @@ final class ExecutableLinesFindingVisitorTest extends TestCase
         $branch = 0;
 
         foreach ($linesFromSource as $lineNumber => $line) {
-            if (false !== strpos($line, 'LINE_ADDED_IN_TEST')) {
+            if (str_contains($line, 'LINE_ADDED_IN_TEST')) {
                 $expected[1 + $lineNumber] = $branch;
 
                 continue;
@@ -94,7 +101,7 @@ final class ExecutableLinesFindingVisitorTest extends TestCase
 
         $this->assertEquals(
             $expected,
-            $executableLinesGroupedByBranch
+            $executableLinesGroupedByBranch,
         );
     }
 }
